@@ -10,6 +10,8 @@ use HTTP::Headers;
 
 use Moose;
 
+use namespace::clean -except => 'meta';
+
 with 'MooseX::Emulate::Class::Accessor::Fast';
 
 has action => (is => 'rw');
@@ -119,8 +121,6 @@ has hostname => (
 
 has _path => ( is => 'rw', predicate => '_has_path', clearer => '_clear_path' );
 
-no Moose;
-
 sub args            { shift->arguments(@_) }
 sub body_params     { shift->body_parameters(@_) }
 sub input           { shift->body(@_) }
@@ -210,6 +210,9 @@ For example, if your action was
 and the URI for the request was C<http://.../foo/moose/bah>, the string C<bah>
 would be the first and only argument.
 
+Arguments just get passed through and B<don't> get unescaped automatically, so
+you should do that explicitly.
+
 =head2 $req->args
 
 Shortcut for arguments.
