@@ -1,7 +1,7 @@
 package TestApp::Controller::Dump;
 
 use strict;
-use base 'Catalyst::Base';
+use base 'Catalyst::Controller';
 
 sub default : Action Private {
     my ( $self, $c ) = @_;
@@ -10,8 +10,7 @@ sub default : Action Private {
 
 sub env : Action Relative {
     my ( $self, $c ) = @_;
-    $c->stash( env => \%ENV );
-    $c->forward('TestApp::View::Dump');
+    $c->forward('TestApp::View::Dump', [\%ENV]);
 }
 
 sub parameters : Action Relative {
